//+------------------------------------------------------------------+
//|                       TriangularMA centered asymmetric bands.mq4 |
//|                                                           mladen |
//| arrowse coded acording to idea presented by umesh                |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1  DimGray
#property indicator_color2  Red
#property indicator_color3  LimeGreen
#property indicator_color4  Blue
#property indicator_color5  Red
#property indicator_style1  STYLE_DOT
#property indicator_width4  2
#property indicator_width5  2

//
//
//
//
//

extern string TimeFrame       = "current time frame";
extern double HalfLength      = 12;
extern int    Price           = PRICE_WEIGHTED;
extern int    AtrLength       = 20;
extern double AtrMultiplier   = 2.0;
extern bool   Interpolate     = true;
extern bool   ShowArrows      = true;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = false;
extern bool   alertsOnHighLow = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double tmBuffer[];
double upBuffer[];
double dnBuffer[];
double upArrow[];
double dnArrow[];

//
//
//
//
//

string IndicatorFileName;
bool   calculatingTma = false;
bool   returningBars  = false;
int    timeFrame;
int    atrTimeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   
   atrTimeFrame  =  PERIOD_D1;
   if (timeFrame >= atrTimeFrame)
   switch (timeFrame)
   {
   case PERIOD_D1: atrTimeFrame = PERIOD_W1; break;
   default:        atrTimeFrame = PERIOD_MN1;
   }
   
   timeFrame  = stringToTimeFrame(TimeFrame);
   HalfLength = MathMax(HalfLength,1);
   IndicatorBuffers(5);
         SetIndexBuffer(0,tmBuffer); SetIndexDrawBegin(0,HalfLength);
         SetIndexBuffer(1,upBuffer); SetIndexDrawBegin(1,HalfLength);
         SetIndexBuffer(2,dnBuffer); SetIndexDrawBegin(2,HalfLength);
         SetIndexBuffer(3,dnArrow);
         SetIndexBuffer(4,upArrow);
         
         //
         //
         //
         //
         //
         
         if (TimeFrame == "calculateTma")  { calculatingTma = true; return(0); }
         if (TimeFrame == "returnBars")    { returningBars  = true; return(0); }  
         
         if (ShowArrows)
         {
           SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,159);
           SetIndexStyle(4,DRAW_ARROW); SetIndexArrow(4,159);
         }
         else
         {
           SetIndexStyle(3,DRAW_NONE);
           SetIndexStyle(4,DRAW_NONE);
         }       
   
   IndicatorFileName = WindowExpertName();
   return(0);
}
int deinit() { return(0); }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-1,Bars-counted_bars+HalfLength);

           if (returningBars)  { tmBuffer[0] = limit; return(0); }
           if (calculatingTma) { calculateTma(limit); return(0); }
           if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));

   //
   //
   //
   //
   //
   
 	for(i = limit; i >= 0; i--)
   {
      int      shift1 = iBarShift(NULL,timeFrame,Time[i]);
      datetime time1  = iTime    (NULL,timeFrame,shift1);

      //
      //
      //
      //
      //
         
         tmBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,0,shift1);
         upBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,1,shift1);
         dnBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,2,shift1);

         upArrow[i]  = EMPTY_VALUE;
         dnArrow[i]  = EMPTY_VALUE;            
            if (High[i+1]>upBuffer[i+1] && Close[i+1]>Open[i+1] && Close[i]<Open[i]) upArrow[i] = High[i]+iATR(NULL,0,20,i)/2.0;
            if ( Low[i+1]<dnBuffer[i+1] && Close[i+1]<Open[i+1] && Close[i]>Open[i]) dnArrow[i] = Low[i] -iATR(NULL,0,20,i)/2.0;

         if (timeFrame <= Period() || shift1==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

      //
      //
      //
      //
      //

         for(int n = 1; i+n < Bars && Time[i+n] >= time1; n++) continue;
         double factor = 1.0 / n;
         for(int k = 1; k < n; k++)
            {
               tmBuffer[i+k] = k*factor*tmBuffer[i+n] + (1.0-k*factor)*tmBuffer[i];
               upBuffer[i+k] = k*factor*upBuffer[i+n] + (1.0-k*factor)*upBuffer[i];
               dnBuffer[i+k] = k*factor*dnBuffer[i+n] + (1.0-k*factor)*dnBuffer[i];
            }               
   }

   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
            int forBar = 0;
      else      forBar = 1;

      int timeBar = iBarShift(NULL,0,iTime(NULL,timeFrame,forBar));
      double up0  = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,1,forBar);
      double dn0  = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,2,forBar);
      double up1  = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,1,forBar+1);
      double dn1  = iCustom(NULL,timeFrame,IndicatorFileName,"calculateTma",HalfLength,Price,AtrLength,AtrMultiplier,2,forBar+1);

      //
      //
      //
      //
      //
      
      if (alertsOnHighLow)       
      {
         double hi0 = iHigh(NULL,timeFrame,forBar); double hi1 = iHigh(NULL,timeFrame,forBar+1);
         double lo0 = iLow( NULL,timeFrame,forBar); double lo1 = iLow( NULL,timeFrame,forBar+1);

            if (hi0 > up0 && hi1 < up1) doAlert(timeBar,"high penetrated upper bar");
            if (lo0 < dn0 && lo1 > dn1) doAlert(timeBar,"low penetrated lower bar");
      }
      else
      {
         double cl0 = iClose(NULL,timeFrame,forBar);
         double cl1 = iClose(NULL,timeFrame,forBar);

            if (cl0 > up0 && cl1 < up1) doAlert(timeBar,"close penetrated upper bar");
            if (cl0 < dn0 && cl1 > dn1) doAlert(timeBar,"close penetrated lower bar");
      }
   } 

   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void calculateTma(int limit)
{
   int i,j,k;
   
   //
   //
   //
   //
   //
   
   for (i=limit; i>=0; i--)
   {
      int      r  = iBarShift(NULL,atrTimeFrame,Time[i]); 
      double atr  = iATR(NULL,atrTimeFrame,AtrLength,r);
      double sum  = (HalfLength+1)*iMA(NULL,0,1,0,MODE_SMA,Price,i);
      double sumw = (HalfLength+1);
      for (j=1, k=HalfLength; j<=HalfLength; j++, k--)
      {
         sum  += k*iMA(NULL,0,1,0,MODE_SMA,Price,i+j);
         sumw += k;

         if (j<=i)
         {
           sum  += k*iMA(NULL,0,1,0,MODE_SMA,Price,i-j);
           sumw += k;
         }
      }
      
      tmBuffer[i] = sum/sumw;
      upBuffer[i] = tmBuffer[i] + atr * AtrMultiplier/10;
      dnBuffer[i] = tmBuffer[i] - atr * AtrMultiplier/10;

      //
      //
      //
      //
      //
            
         
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="";
   static datetime previousTime;
   string message;

   //
   //
   //
   //
   //
   
   if (previousAlert!=doWhat || previousTime!=Time[forBar]) 
   {
      previousAlert = doWhat;
      previousTime  = Time[forBar];

      message= StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," THA : ",doWhat);
         if (alertsMessage) Alert(message);
         if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"TMA "),message);
         if (alertsSound)   PlaySound("alert2.wav");
    }
}

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   for(int l = StringLen(tfs)-1; l >= 0; l--)
   {
      int char = StringGetChar(tfs,l);
          if((char > 96 && char < 123) || (char > 223 && char < 256))
               tfs = StringSetChar(tfs, l, char - 32);
          else 
              if(char > -33 && char < 0)
                  tfs = StringSetChar(tfs, l, char + 224);
   }
   int tf=0;
         if (tfs=="M1" || tfs=="1")     tf=PERIOD_M1;
         if (tfs=="M5" || tfs=="5")     tf=PERIOD_M5;
         if (tfs=="M15"|| tfs=="15")    tf=PERIOD_M15;
         if (tfs=="M30"|| tfs=="30")    tf=PERIOD_M30;
         if (tfs=="H1" || tfs=="60")    tf=PERIOD_H1;
         if (tfs=="H4" || tfs=="240")   tf=PERIOD_H4;
         if (tfs=="D1" || tfs=="1440")  tf=PERIOD_D1;
         if (tfs=="W1" || tfs=="10080") tf=PERIOD_W1;
         if (tfs=="MN" || tfs=="43200") tf=PERIOD_MN1;
         if (tf==0 || tf<Period())      tf=Period();
   return(tf);
} 
  